<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       http://www.welaunch.io
 * @since      1.0.0
 *
 * @package    WooCommerce_Quick_Order
 * @subpackage WooCommerce_Quick_Order/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    WooCommerce_Quick_Order
 * @subpackage WooCommerce_Quick_Order/public
 * @author     Daniel Barenkamp <support@welaunch.io>
 */
class WooCommerce_Quick_Order_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * options of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      array    $options
	 */
	private $options;

	/**
	 * if true this product will be excluded
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      array    $options
	 */
	private $exclude;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) 
	{
		$this->plugin_name = $plugin_name;
		$this->version = $version;
	}

	/**
	 * Enqueu Styles
	 * @author Daniel Barenkamp
	 * @version 1.0.0
	 * @since   1.0.0
	 * @link    https://www.welaunch.io
	 * @return  [type]                       [description]
	 */
	public function enqueue_styles() 
	{
		global $woocommerce_quick_order_options;

		$this->options = $woocommerce_quick_order_options;

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/woocommerce-quick-order-public.css', array(), $this->version, 'all' );
		if($this->get_option('datatablesEnable')) {

			wp_enqueue_style( 'jquery-datatables', plugin_dir_url( __FILE__ ) . 'css/vendor/jquery.dataTables.min.css', array(), '1.10.21', 'all' );

			if($this->get_option('datatablesResponsive')) {
				wp_enqueue_style( 'jquery-datatables-responsive', plugin_dir_url( __FILE__ ) . 'css/vendor/responsive.dataTables.min.css', array(), '2.2.5', 'all' );
			}
			
			if($this->get_option('datatablesPrint') || $this->get_option('datatablesCopyHtml5') || $this->get_option('datatablesExcelHtml5') || $this->get_option('datatablesCsvHtml5') || $this->get_option('datatablesPdfHtml5')) {
				wp_enqueue_style( 'jquery-datatables-buttons', plugin_dir_url( __FILE__ ) . 'css/vendor/buttons.dataTables.min.css', array(), '1.6.2', 'all' );
			}

			if($this->get_option('datatablesFixedHeader')) {
				wp_enqueue_style( 'jquery-datatables-fixedheader', plugin_dir_url( __FILE__ ) . 'css/vendor/fixedHeader.dataTables.min.css', array(), '3.1.7', 'all' );
			}
		}

		if($this->get_option('lightboxEnable')) {
			wp_enqueue_style( 'luminous-lightbox', plugin_dir_url( __FILE__ ) . 'css/vendor/luminous-basic.min.css', array(), '2.3.2', 'all' );
		}
	}

	/**
	 * Enque JS SCripts
	 * @author Daniel Barenkamp
	 * @version 1.0.0
	 * @since   1.0.0
	 * @link    https://www.welaunch.io
	 * @return  [type]                       [description]
	 */
	public function enqueue_scripts() 
	{
		global $woocommerce_quick_order_options;

		$this->options = $woocommerce_quick_order_options;

		if($this->get_option('datatablesEnable')) {
			wp_enqueue_script( 'jquery-datatables', plugin_dir_url( __FILE__ ) . 'js/vendor/jquery.dataTables.min.js', array( 'jquery' ), '1.10.21', true );

			$jss = array();
			
			if($this->get_option('datatablesResponsive')) {
				$jss['datatables-responsive'] = plugin_dir_url( __FILE__ ) . 'js/vendor/dataTables.responsive.min.js';
			}

			if($this->get_option('datatablesPrint') || $this->get_option('datatablesCopyHtml5') || $this->get_option('datatablesExcelHtml5') || $this->get_option('datatablesCsvHtml5') || $this->get_option('datatablesPdfHtml5')) {
				$jss['datatables-buttons'] = plugin_dir_url( __FILE__ ) . 'js/vendor/dataTables.buttons.min.js';
			}

			if($this->get_option('datatablesPdfHtml5')) {
				$jss['datatables-pdfmake'] = plugin_dir_url( __FILE__ ) . 'js/vendor/pdfmake.min.js';
				$jss['datatables-vfs_fonts'] = plugin_dir_url( __FILE__ ) . 'js/vendor/vfs_fonts.js';
			}

			if($this->get_option('datatablesCopyHtml5') || $this->get_option('datatablesExcelHtml5') || $this->get_option('datatablesCsvHtml5') || $this->get_option('datatablesPdfHtml5')) {
				$jss['datatables-jszip'] = plugin_dir_url( __FILE__ ) . 'js/vendor/jszip.min.js';
				$jss['datatables-html5'] = plugin_dir_url( __FILE__ ) . 'js/vendor/buttons.html5.min.js';
			}

			if($this->get_option('datatablesPrint')) {
				$jss['datatables-print'] = plugin_dir_url( __FILE__ ) . 'js/vendor/buttons.print.min.js';
			}

			if($this->get_option('datatablesFixedHeader')) {
				$jss['datatables-fixedheader'] = plugin_dir_url( __FILE__ ) . 'js/vendor/dataTables.fixedHeader.min.js';
			}		

			foreach ($jss as $key => $js) {
				wp_enqueue_script( $key, $js, array( 'jquery', 'jquery-datatables' ), '1.10.21', true );
			}
		}

		if($this->get_option('lightboxEnable')) {
			wp_enqueue_script( 'luminous-lightbox', plugin_dir_url( __FILE__ ) . 'js/vendor/luminous.min.js', array( 'jquery' ), '2.3.2', true );
		}

		wp_enqueue_script( $this->plugin_name . '-public', plugin_dir_url( __FILE__ ) . 'js/woocommerce-quick-order-public.js', array( 'jquery'), $this->version, true );
        $forJS['ajax_url'] = admin_url('admin-ajax.php');
        $forJS['lightbox'] = $this->get_option('lightboxEnable');
        $forJS['trans'] = array(
        	'showAll' => __('Show All', 'woocommerce-quick-order'),
        	'searchText' => __('Search', 'woocommerce-quick-order'),
        	'sku' => __('SKU: ', 'woocommerce-quick-order'),
        	'price' => __('Price: ', 'woocommerce-quick-order'),
        	'placeholder' => __('Enter SKU or Product Name ...', 'woocommerce-quick-order'),
        	'addedToCart' => __('Added to Cart', 'woocommerce-quick-order'),
        	'outofstock' => __('Out of stock', 'woocommerce'),
    	);

        $numericSortingColumns = array();
        $priceColumnPosition = array_search('pr', array_keys($this->get_option('dataToShowData')['enabled']));
        if(!empty($priceColumnPosition)) {
        	$numericSortingColumns[] = $priceColumnPosition;
        }
		$forJS['numericSortingColumns'] = $numericSortingColumns;

    	$forJS = apply_filters('woocommerce_quick_order_js_settings', $forJS);
        wp_localize_script($this->plugin_name . '-public', 'woocommerce_quick_order_options', $forJS);
	}

	/**
	 * Get Options
	 * @author Daniel Barenkamp
	 * @version 1.0.0
	 * @since   1.0.0
	 * @link    https://www.welaunch.io
	 * @param   [type]                       $option [description]
	 * @return  [type]                               [description]
	 */
    private function get_option($option)
    {
    	if(!is_array($this->options)) {
    		return false;
    	}
    	
    	if(!array_key_exists($option, $this->options))
    	{
    		return false;
    	}
    	return $this->options[$option];
    }
	
    /**
     * Inits the Quick Order
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https://www.welaunch.io
     * @return  [type]                       [description]
     */
    public function init()
    {

		global $woocommerce_quick_order_options;
		$this->options = $woocommerce_quick_order_options;
		if (!$this->get_option('enable'))
		{
			return false;
		}

		$excludeUserRoles = $this->get_option('applyForExcludeUserRoles');
		if(is_array($excludeUserRoles) && !empty($excludeUserRoles)) {

			$currentUserRole = $this->get_user_role();
			if(in_array($currentUserRole, $excludeUserRoles)) {
				return false;
			}
		}

		// Only not logged in
		if($this->get_option('applyForUserGroup') == 2) {
			if(is_user_logged_in()) {
				return false;
			}
		}

		// Only logged in
		if($this->get_option('applyForUserGroup') == 3) {
			if(!is_user_logged_in()) {
				return false;
			}
		}

		$tableMetaDataTexts = $this->get_option('tableMetaDataTexts');
		if(!empty($tableMetaDataTexts)) {

			$temp = array();
			foreach($tableMetaDataTexts as $variationsTableMetaDataText) {
				$tableMetaDataTextSplit = explode('|', $variationsTableMetaDataText);
				if(empty($tableMetaDataTextSplit[0]) || empty($tableMetaDataTextSplit[1])) {
					continue;
				}
				$temp[$tableMetaDataTextSplit[0]] = $tableMetaDataTextSplit[1];
			}
			$this->tableMetaDataTexts = $temp;
		}

		add_shortcode( 'woocommerce_quick_order_search', array($this, 'get_search'));
		add_shortcode( 'woocommerce_quick_order_table', array($this, 'get_table'));
    }

    /**
     * Check if apply on single Product
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    https://www.welaunch.io
     * @return  [type]                       [description]
     */
    public function get_search($atts)
    {
		$args = shortcode_atts( array(
	        'only_on_stock' => '',
	        'show_stock' => '',
	        'categories' => '',
	    ), $atts );

		$options = array(
			'only_on_stock' => false,
			'show_stock' => false,
			'categories' => array(),
		);

		if(!empty($args['categories'])) {
	    	$options['categories'] = $args['categories'];
    	}

		if(!empty($args['only_on_stock'])) {
	    	$options['only_on_stock'] = $args['only_on_stock'];
    	} 

		if(!empty($args['show_stock'])) {
	    	$options['show_stock'] = $args['show_stock'];
    	} 


    	ob_start();
    	?>
    	<table id="woocommerce-quick-order-search" class="woocommerce-quick-order-search nowrap rwd-table" width="100%" data-options="<?php echo htmlspecialchars( json_encode( $options), ENT_QUOTES, 'UTF-8') ?>">
	 		<thead>
	            <tr>
					<th><?php echo __('Product', 'woocommerce-quick-order') ?></th>
					<th><?php echo __('Price', 'woocommerce-quick-order') ?></th>
					<th><?php echo __('Quantity', 'woocommerce-quick-order') ?></th>
					<th></th>
	            </tr>
	        </thead>
			<tbody>
				<tr>
					<td data-th="<?php echo __('Product', 'woocommerce-quick-order') ?>" class="woocommerce-quick-order-search-input">
						<div class="woocommerce-quick-order-search-input-indication"></div>
						<input type="text" name="woocommerce-quick-order-search-input-field" class="woocommerce-quick-order-search-input-field" 
						placeholder="<?php echo __('Enter SKU or Product Name ...', 'woocommerce-quick-order') ?>">
					</td>
					<td data-th="<?php echo __('Price', 'woocommerce-quick-order') ?>" class="woocommerce-quick-order-search-price">
						–
					</td>
					<td data-th="<?php echo __('Quantity', 'woocommerce-quick-order') ?>" class="woocommerce-quick-order-search-quantity">
						<input type="number" name="woocommerce-quick-order-quantity-field" class="woocommerce-quick-order-search-quantity-field" value="1">
					</td>
					<td class="woocommerce-quick-order-search-actions">
					</td>
				</tr>
	        </tbody>
	    </table>
	    <a href="#" class="woocommerce-quick-order-search-add-row">
	    	<i class="fa fa-plus"></i> <?php echo __('Add another Row', 'woocommerce-quick-order') ?>
	    </a>
	    <div class="woocommerce-quick-order-search-add-to-cart">
	    	<a href="#" class="woocommerce-quick-order-search-add-to-cart-btn button product_type_simple add_to_cart_button ajax_add_to_cart btn theme-button"><?php echo __('Add all to Cart', 'woocommerce-quick-order') ?></a>
	    </div>
		<?php
	   	$html = ob_get_contents();
		ob_end_clean();
		return $html;
    }

    public function get_table($atts)
	{
		$html = "";
		$this->products_in = "";
		$this->categories_in = "";

		$args = shortcode_atts( array(
	        'products' => '',
	        'categories' => '',
	        'order' => 'DESC',
	        'orderby' => 'menu_order',
	        'splitcategories' => '0',
	    ), $atts );

		if(!empty($args['products'])) {
	    	$products_in = explode(',', $args['products']);
	    	$this->products_in = $products_in;
	    } elseif(!empty($args['categories'])) {
	    	$categories_in = explode(',', $args['categories']);
	    	$this->categories_in = $categories_in;
	    }

	    $this->order = $args['order'];
	    $this->orderby = $args['orderby'];

	    if($args['splitcategories'] == "1" && !empty($this->categories_in)) {   	

	    	foreach ($this->categories_in as $category_id) {

		    	$term = get_term($category_id);
		    	if(!isset($term->name)) {
		    		continue;
		    	}

	    		$this->categories_in = $category_id;

				$products = $this->get_products();

			    if(empty($products)) {
			    	return _e('No products found', 'woocommerce-quick-order');
			    }

			    ob_start();

			    $this->products = $products;

			    $this->render_table();

			    $html .= '<h2 class="woocommerce-quick-order-split-categories-header">' . $term->name . '</h2>';

			   	$html .= ob_get_contents();
				
			   	ob_end_clean();
	    	}


    	} else {

		    $products = $this->get_products();

		    if(empty($products)) {
		    	return _e('No products found', 'woocommerce-quick-order');
		    }

		    $this->products = $products;
		    
		    ob_start();

		    $this->render_table();
		    
		   	$html = ob_get_contents();
			ob_end_clean();
		}

		return $html;
    }

    public function render_table()
    {
  		$this->dataToShowData = apply_filters('woocommerce_quick_order_table_data', $this->get_option('dataToShowData') );

  		$this->dataToShowData = $this->dataToShowData['enabled'];
  		unset($this->dataToShowData['placebo']);

  		$this->first = true;

  		$this->get_table_start();
		$this->get_table_header();
		$this->get_table_rows();
		$this->get_table_footer();

		do_action('woocommerce_quick_order_before_table');

		echo $this->table_start;
		echo $this->header;
		echo $this->rows;
		echo $this->tfooter;

		do_action('woocommerce_quick_order_after_table');

    }

    public function get_table_footer()
    {
    	$this->tfooter = "";
    	if(array_key_exists('mc', $this->dataToShowData) || array_key_exists('mq', $this->dataToShowData)) {

    		$this->tfooter .= '<tfoot>';
    			$this->tfooter .= '<tr id="quick-order-table-multiple-add-to-cart" class="quick-order-table-multiple-add-to-cart">';
    				$this->tfooter .= '<td colspan="' . $this->columnCount . '">';
						$this->tfooter .= '<button type="button" disabled href="#" id="quick-order-table-multiple-add-to-cart-btn" class="quick-order-table-multiple-add-to-cart-btn btn button btn-primary btn-lg">' . __('Add all to Cart', 'woocommerce-quick-order') . '</button>';
					$this->tfooter .= '</td>';
				$this->tfooter .= '</tr>';
			$this->tfooter .= '</tfoot>';
    	}
    	$this->tfooter .= '</table></div>';
    	return $this->tfooter;
    }

    public function get_table_start()
    {
		$datatablesEnabled = apply_filters('woocommerce_quick_order_datatables_enabled', $this->get_option('datatablesEnable'));

		if($datatablesEnabled == "1") {

			$datatablesOptions = array(
				'paging' => $this->get_option('datatablesPaging') == "1" ? true : false,
				'pageLength' => (int) $this->get_option('datatablesPageLength') ? (int) $this->get_option('datatablesPageLength') : 10,
				'lengthMenu' => $this->get_option('datatablesPageLengthMenu') ? explode(',', $this->get_option('datatablesPageLengthMenu') ) : array( 10, 25, 50, 60, 75, 100 ),
				
				'orderColumn' => !empty($this->get_option('datatablesOrderColumn')) ? $this->get_option('datatablesOrderColumn') : '',
				'orderColumnType' => !empty($this->get_option('datatablesOrderColumnType')) ? $this->get_option('datatablesOrderColumnType') : '',
				'ordering' => $this->get_option('datatablesOrdering') == "1" ? true : false,
				'info' => $this->get_option('datatablesInfo') == "1" ? true : false,
				'stateSave' => $this->get_option('datatablesStateSave') == "1" ? true : false,
				'searching' => $this->get_option('datatablesSearching') == "1" ? true : false,
				'fixedHeaderA' => $this->get_option('datatablesFixedHeader'),
				'fixedHeaderOffset' => $this->get_option('datatablesFixedHeaderOffset'),
				'fixedFooterA' => $this->get_option('datatablesFixedFooter'),
				'fixedFooterOffset' => $this->get_option('datatablesFixedFooterOffset'),
				'responsive' => $this->get_option('datatablesResponsive') == "1" ? true : false,
				'filtering' => $this->get_option('datatablesFiltering'),
				'columnSearch' => $this->get_option('datatablesColumnSearch'),
				'orderCellsTop' => $this->get_option('datatablesColumnSearch') == "1" ? true : false,
				'sDom' => $this->get_option('datatablesSDom'),
				'language' => array(
					'url' => '//cdn.datatables.net/plug-ins/9dcbecd42ad/i18n/' . $this->get_option('datatablesLanguage') . '.json',
					// 'url' => '//cdn.datatables.net/plug-ins/9dcbecd42ad/i18n/German.json',
					// 'url' => '//cdn.datatables.net/plug-ins/1.10.21/i18n/' . $this->get_option('datatablesLanguage') . '.json',
				)
			);

			if($this->get_option('datatablesScrollCollapse') == "1") {
				$datatablesOptions['scrollCollapse'] = true;
				$datatablesOptions['scrollY'] = $this->get_option('datatablesScrollY') . 'px';
			}

			$buttons = array();
			if($this->get_option('datatablesPrint') == "1") {
				$buttons[] = array(
					'extend' => 'print',
					'exportOptions' => array(
						'stripHtml' => false,
					)
				);
			}
			if($this->get_option('datatablesCopyHtml5') == "1") {
				$buttons[] = array(
					'extend' => 'copyHtml5',
					'exportOptions' => array(
						'stripHtml' => false,
					)
				);
			}
			if($this->get_option('datatablesExcelHtml5') == "1") {
				$buttons[] = array(
					'extend' => 'excelHtml5',
					'exportOptions' => array(
						'stripHtml' => true,
					)
				);
			}
			if($this->get_option('datatablesCsvHtml5') == "1") {
				$buttons[] = array(
					'extend' => 'csvHtml5',
					'exportOptions' => array(
						'stripHtml' => true,
					)
				);
			}
			if($this->get_option('datatablesPdfHtml5') == "1") {
				$buttons[] = array(
					'extend' => 'pdfHtml5',
					'exportOptions' => array(
						'stripHtml' => true,
					)
				);
			}

			if(!empty($buttons)) {
				$datatablesOptions['buttons'] = $buttons;
			}

			$datatablesOptions = apply_filters('woocommerce_quick_order_datatables_options', $datatablesOptions);

			$css = "";
			if($this->get_option('datatablesSearchingProminent')) {
				$css .= ' woocommerce-quick-order-search-prominent';
			}

			$this->table_start = '<div class="woocommerce-quick-order-table-container' . $css . '">
									<table id="woocommerce-quick-order" class="woocommerce-quick-order datatables nowrap" data-options=' . json_encode($datatablesOptions, true) . ' width="100%">';
		} else {
			$this->table_start = '<div class="woocommerce-quick-order-table-container">
										<table id="woocommerce-quick-order" class="woocommerce-quick-order nowrap rwd-table" width="100%">';
		}

		return $this->table_start;
    }

    public function get_table_rows()
    {
    	global $product;
    	$this->rows = "";

        foreach ($this->products as $product) {

           	$product = wc_get_product($product->ID);

			if($product->is_type( 'variable' )) {
				$available_variations = $product->get_available_variations();
				if(empty($available_variations)) {
					continue;
				}

				foreach ($available_variations as $variation) {
					$variation_product = new WC_Product_Variation( $variation['variation_id'] );

		           	$this->rows .= '<tr id="quick-order-table-row-' . $variation_product->get_id() . '" class="quick-order-table-row">';
		           	foreach($this->dataToShowData as $dataToShowKey => $dataToShowValue) {
		           		$data = $this->get_data($dataToShowKey, $variation_product);
		        		$this->rows .=  '<td data-th="' . __( $dataToShowValue, 'woocommerce-quick-order' ) . '" class="quick-order-table-value-' . $dataToShowKey . ' quick-order-table-value">' . $data . '</td>';
		        	}
		        	$this->rows .= '</tr>';

		        	$this->first = false;
				}
				
			} else {

	           	$this->rows .= '<tr id="quick-order-table-row-' . $product->get_id() . '" class="quick-order-table-row">';
	           	foreach($this->dataToShowData as $dataToShowKey => $dataToShowValue) {
	           		$data = $this->get_data($dataToShowKey, $product);
	        		$this->rows .=  '<td data-th="' . __( $dataToShowValue, 'woocommerce-quick-order' ) . '" class="quick-order-table-value-' . $dataToShowKey . ' quick-order-table-value">' . $data . '</td>';
	        	}
	        	$this->rows .= '</tr>';
        	}
        	
        	$this->first = false;
		}
		wp_reset_postdata();
	}

	public function get_table_header()
	{
		$this->header = "<thead>";
		$this->columnCount = 0;
		foreach($this->dataToShowData as $dataToShowKey => $dataToShowValue) {
           		
			if($dataToShowKey == "mc" || $dataToShowKey == "mq") {
				$dataToShowValue = "";
			}

			if (strpos($dataToShowKey, 'meta_') !== FALSE){ // variation is a pre-definted attribute
				$meta_key = substr($dataToShowKey, 5);
				if(isset($this->tableMetaDataTexts[$meta_key]) && !empty($this->tableMetaDataTexts[$meta_key])) {
					$dataToShowValue = $this->tableMetaDataTexts[$meta_key];
				}
			}

			$this->header .= '<th id="quick-order-table-header-' . $dataToShowKey . '" class="quick-order-table-header">' . __( $dataToShowValue, 'woocommerce-quick-order' ) . '</th>';
			$this->columnCount++;
    	}
    	$this->header .= '</thead>';
	}

	/**
	 * Get Variation data by Key
	 * @author Daniel Barenkamp
	 * @version 1.0.0
	 * @since   1.0.0
	 * @link    https://www.welaunch.io
	 * @param   [type]                       $key       [description]
	 * @param   [type]                       $variation [description]
	 * @return  [type]                                  [description]
	 */
	public function get_data($key, $product)
	{
		if(empty($key)) {
			return;
		}

		$html = "";
		if (strpos($key, 'meta_') !== FALSE){

			$meta_key = substr($key, 5);

			$html = get_post_meta($product->get_id(), $meta_key, true);
			if(empty($html) && $product->is_type('variation')) {
				$html = get_post_meta($product->get_parent_id(), $meta_key, true);
			}
		// Get Attributes
		} elseif(strpos($key, 'attr') !== false) {

			$attribute_slug = substr($key, 5);
			$attribute_value = $product->get_attribute($attribute_slug);
			if($product->get_id() == 93) {

				// var_dump($product->get_attribute($attribute_slug));
				// die();
			}
			if(!empty($attribute_value)) {
				$html = $attribute_value;
			}
		} else {

			switch ($key) {
				case 'im':
					$imageId = $product->get_image_id();
					$image = wp_get_attachment_image($imageId);

					if($this->get_option('lightboxEnable')) {
						$fullImage = wp_get_attachment_image_src($imageId, 'full');
						$html = '<a href="' . $fullImage[0] . '" class="woocommerce-quick-order-lightbox">' . $image . '</a>';
					} else {
						$html = $image; 
					}

					break;
				case 'sk':
					$html = $product->get_sku();
					break;
				case 'pr':
					if($this->get_option('datatablesFiltering')) {
						$html = strip_tags( wc_price( wc_get_price_to_display( $product ) ) );
					} else {
						$html = wc_price( wc_get_price_to_display( $product ) );
					}
					break;
				case 'pt':
					$html = $product->get_name();
					if($this->get_option('dataToShowShowShortDescriptionBelowName')) {
						$html .= '<div class="woocommerce-quick-order-short-description">' . wpautop( $product->get_short_description() ) . '</div>';
					}
					break;
				case 'ct':
					$html = wc_get_product_category_list($product->get_id());
					break;
				case 'st':

					$availability = $product->get_availability();

					if($product->is_on_backorder( 1 )) {
						$html = __('In Stock', 'woocommerce-quick-order');
					} elseif($product->managing_stock()) {
						$html = $availability['availability'];
					} else {
						if($product->is_in_stock()) {
							$html = __('In Stock', 'woocommerce-quick-order');
						} else {
							$html = __('Out of Stock', 'woocommerce-quick-order');
						}
					}
					break;
				case 'de':
					$html = wpautop( $product->get_short_description() );
					break;
				case 'di':
					$html = wc_format_dimensions( $product->get_dimensions(false) );
					break;
					break;
				case 'we':
					$html = wc_format_weight( $product->get_weight() );
					break;
				case 'rm':
					$link = get_permalink($product->get_id());
					$html = '<a target="_blank" class="button btn theme-button" href="' . $link . '">' . __('Read More', 'woocommerce-quick-order') . '</a>';
					break;
				case 'ca':
					// ob_start();
					$html = $this->get_add_to_cart($product);
					// do_action('woocommerce_simple_add_to_cart');

					// $html = ob_get_contents();
					// ob_end_clean();
					break;
				case 'qt':
					$html = $this->get_quantity($product);
					break;
				case 'eq':
					$html = $this->get_enquiry_button($product);
					break;
				case 'pd':
					$html = $this->get_pdf_button($product);
					break;
				case 'wi':
					if($this->get_option('wishlistYith')) {
						$html = do_shortcode('[yith_wcwl_add_to_wishlist product_id= ' . $product->get_id() . ' ]');
					} else {
						$html = do_shortcode('[woocommerce_wishlist_button product= ' . $product->get_id() . ' ]');
					}
					break;
				case 'mc':
					$product_id = $product->get_id();
					$html = '<input type="checkbox" class="quick-order-table-multiple-add-to-cart-products" name="quick-order-table-multiple-add-to-cart-products" data-product_id="' . $product_id . '">';
					break;
				case 'mq':
				
					$product_id = $product->get_id();
					$html = '<div class="quick-order-table-multiple-quantity-add-to-cart-products" data-product_id="' . $product_id . '">';
						$html .= $this->get_multiple_quantity($product);
					$html .= '</div>';
					break;
				default:
					
					break;
			}
		}

		return $html;
	}

	/**
	 * Get Variation Add to Cart
	 * @author Daniel Barenkamp
	 * @version 1.0.0
	 * @since   1.0.0
	 * @link    https://www.welaunch.io
	 * @param   [type]                       $variation [description]
	 * @return  [type]                                  [description]
	 */
	public function get_add_to_cart($product)
	{

		ob_start();

		if($product->is_type('variation')) {
			$variations = $product->get_attributes();
			$variation = $product;
			$variation_string = "";
			if(!empty($variations)) {
				foreach ($variations as $key => $value) {
					$variation_string .= $key . '=' . $value . ',';
				}
				$variation_string = rtrim($variation_string, ',');
			}
			?>
			
			<div class="woocommerce-variation-add-to-cart woocommerce-quick-order-table-add-to-cart variations_button">
				<button type="submit" class=" button alt"><?php echo esc_html( $product->single_add_to_cart_text() ); ?></button>
				<input type="hidden" name="add-to-cart" value="<?php echo absint( $product->get_parent_id() ); ?>" />
				<input type="hidden" name="product_id" value="<?php echo absint( $product->get_parent_id() ); ?>" />
				<input type="hidden" name="variation_id" class="variation_id" value="<?php echo $variation->get_id() ?>" />
				<input type="hidden" name="variation" class="variation" value="<?php echo $variation_string ?>" />
			</div>
			
			<?php
		} else {
			?>
			<div class="woocommerce-variation-add-to-cart woocommerce-quick-order-table-add-to-cart">
				<button type="submit" name="add-to-cart" value="<?php echo $product->get_id() ?>" class="woocommerce-quick-order-single-add-to-cart-button single_add_to_cart_button button alt">
					<?php echo esc_html( $product->single_add_to_cart_text() ); ?>
				</button>
			</div>
			<?php
		}

		$html = ob_get_contents();
		ob_end_clean();
		return $html;
	}

	public function get_enquiry_button($product)
	{
		if(!class_exists('WooCommerce_Catalog_Mode')) {
			return;
		}

		global $woocommerce_catalog_mode_options;

		$buttonText = apply_filters('woocommerce_catalog_mode_enquiry_button_text', $woocommerce_catalog_mode_options['singleProductButtonText'] );

    	$product_data = array(
    		'name' => $product->get_name(),
    		'sku' => $product->get_sku(),
    	);
    	$btnExtra = "data-products='" . json_encode($product_data). "'";

  		ob_start();
		?>
		<button type="button" class="enquiryLoopButton btn button btn-primary btn-lg" <?php echo $btnExtra ?>>
			<?php echo $buttonText ?>
		</button>
  		
		<?php

		$html = ob_get_contents();
		ob_end_clean();
		return $html;
	}

	public function get_pdf_button($product)
	{
		if(!class_exists('WooCommerce_Print_Products')) {
			return;
		}

		global $woocommerce_print_products_options;

		$buttonText = $woocommerce_print_products_options['iconTypeButtonPDFText'];

		if($product->is_type('variation')) {

			$parentProductID = $product->get_parent_id();
			$parentProductURL = get_permalink($parentProductID);
			$productPDFExportURL = $parentProductURL . '?print-products=pdf&variation=' . $product->get_id();

		} else {
			$productURL = get_permalink($product->get_id());
			$productPDFExportURL = $productURL . '?print-products=pdf';
		}

  		ob_start();
		?>
		<a type="button" href="<?php echo $productPDFExportURL ?>" target="_blank" class="woocommerce-quick-order-pdf-export-button btn button btn-primary btn-lg">
			<?php echo $buttonText ?>
		</a>
  		
		<?php

		$html = ob_get_contents();
		ob_end_clean();
		return $html;
	}

    public function get_products()
    {
	    $args = array(
		    'post_type'             => 'product',
		    'post_status'           => 'publish',
		    'ignore_sticky_posts'   => 1,
		    'posts_per_page'        => -1,
		    'post__in'				=> $this->products_in,
		    'order'					=> $this->order,
		    'orderby'				=> $this->orderby,
		);
		if(!empty($this->categories_in)) {
			$args['tax_query'] = array(
		        array(
		            'taxonomy'      => 'product_cat',
		            'field' 		=> 'term_id', //This is optional, as it defaults to 'term_id'
		            'terms'         => $this->categories_in,
		            'operator'      => 'IN' // Possible values are 'IN', 'NOT IN', 'AND'.
		        ),
	        );
		}
	    $products = new WP_Query($args);

	    return $products->posts;
    }


   	public function search_products()
   	{
   		global $wpdb;




   		$response = array(
   			'message' => __('No Products found ...', 'woocommerce-quick-order'),
   			'products' => array(),
   		);

   		$skuOrProduct = $_POST['term'];
   		if(empty($skuOrProduct)) {
   			die(json_encode($response));
   		}

   		$args = array(
   			'posts_per_page'  => 50,
            'post_type'       => array('product', 'product_variation'),
            'meta_query' => array(
            	'relation' => 'AND',
            ),
   		);

   		$args['meta_query']['sku'] = array(
            'key' => '_sku',
            'value' => $skuOrProduct,
            'compare' => 'LIKE'
        );

   		$categoriesToCheck = array();
        if(isset($_POST['options']) && isset($_POST['options']['categories']) && !empty($_POST['options']['categories'])) {

        	$categoriesToCheck = explode(',', $_POST['options']['categories']);
        	if(!empty($categoriesToCheck)) {

				$args['tax_query'] = array(
	        		array(
	            		'taxonomy'  => 'product_cat',
	            		'field'     => 'id', 
	            		'terms'     => $categoriesToCheck,
	        		)
	    		);
    		}
        }

        if(isset($_POST['options']) && isset($_POST['options']['only_on_stock']) && !empty($_POST['options']['only_on_stock']) && $_POST['options']['only_on_stock'] == "yes") {
        	
        	$args['meta_query'][] = array(
	            'key' => '_stock_status',
	            'value' => 'instock'
	        );

        	$args['meta_query'][] = array(
	            'key' => '_backorders',
	            'value' => 'no'
	        );

        }

        // var_dump($args);
        

   		// $bySKU = wc_get_product_id_by_sku($skuOrProduct);
		// $products = $wpdb->get_results( $wpdb->prepare( "SELECT post_id as ID FROM $wpdb->postmeta WHERE meta_key = '_sku' AND meta_value LIKE %s", 
		// 	'%' . $skuOrProduct . '%') );

   		// if(empty($products)) {
   		// 	$products = $this->search_product_by_name($skuOrProduct);
   		// } 
		$products = get_posts( $args );

		if(empty($products)) {
			unset($args['meta_query']['sku']);
			$args['s'] = $skuOrProduct;	
			$products = get_posts( $args );
		}

   		if(!empty($products)) {
			$temp = array();
	    	foreach ($products as $prod) {

	    		$product_id = $prod->ID;
	    		$product = wc_get_product($product_id);

	    		// if(!empty($categoriesToCheck)) {
	    		// 	var_dump($categoriesToCheck);
	    		// 	var_dump( has_term( $categoriesToCheck, 'product_cat',  $prod) );
	    		// 	var_dump($product->get_category_ids());
	    		// 	die();
	    		// 	if(!has_term( $categoriesToCheck, 'product_cat',  $prod)) {
	    		// 		continue;
	    		// 	}
	    		// }


	    		if($product->is_type( 'variable' )) {

					$available_variations = $product->get_available_variations();
					if(empty($available_variations)) {
						continue;
					}

					foreach ($available_variations as $variation) {
						$variation_product = new WC_Product_Variation( $variation['variation_id'] );
						$temp[] = array(
			    			'ID' => $variation_product->get_id(),
			    			'name' => $variation_product->get_name(),
							'img' => get_the_post_thumbnail_url($variation_product->get_id()),
							'sku' => $variation_product->get_sku(),
							'price' => $variation_product->get_price() . ' ' . get_woocommerce_currency_symbol(),
							'price_display' => wc_price($variation_product->get_price() ),
							'stock_status' => $variation_product->get_stock_status(),
							'stock_status_html' => wc_get_stock_html( $variation_product ),
			    		);
					}
					
    			} else {
		    		$temp[] = array(
		    			'ID' => $product->get_id(),
		    			'name' => $product->get_name(),
						'img' => get_the_post_thumbnail_url($product->get_id()),
						'sku' => $product->get_sku(),
						'price' => $product->get_price() . ' ' . get_woocommerce_currency_symbol(),
						'price_display' => wc_price($product->get_price() ),
						'stock_status' => $product->get_stock_status(),
						'stock_status_html' => wc_get_stock_html( $product ),
		    		);
	    		}
	    	}
	    	$products = $temp;

   			$response['message'] = sprintf( __('%d products found!', 'woocommerce-quick-order'), count($products));
   			$response['products'] = $products;
   		}
   		
   		die(json_encode($response));
   	}

   	protected function search_product_by_name($title)
   	{
	    global $wpdb;
	    $title = esc_sql($title);

	    if(!$title) {
	    	return false;
	    }
	    
	    $products = $wpdb->get_results("
	        SELECT ID
	        FROM $wpdb->posts
	        WHERE post_title LIKE '%$title%'
	        AND post_type = 'product' 
	        AND post_status = 'publish'
	    ");
	    if(empty($products)) {
	    	return false;
	    }

	    return $products;
   	}
	
	public function add_to_cart()
	{
		ob_start();

		$cart_item_data = array();		

	    $product_id        = apply_filters( 'woocommerce_add_to_cart_product_id', absint( $_POST['product_id'] ) );
	    $quantity          = empty( $_POST['quantity'] ) ? 1 : wc_stock_amount( $_POST['quantity'] );

	    $variation_id      = isset( $_POST['variation_id'] ) ? $_POST['variation_id'] : '';
	   	
	   	if(isset($_POST['variation'])) {
	        $attributes   = explode(',', $_POST['variation']);
	        $variation    = array();
	        if(!empty($attributes)) {
	        	foreach($attributes as $values){
	            	$values = explode('=', $values);
	            	if($this->get_option('oldThemeSupport')) {
						$variation[$values[0]] = $values[1];
	            	} else {
	            		$variation['attribute_' . $values[0]] = $values[1];
	            	}	
	        	}
	        }
        }

	    $passed_validation = apply_filters( 'woocommerce_add_to_cart_validation', true, $product_id, $quantity, $variation_id, $variation, $cart_item_data );
		if ( $passed_validation) {
			$added = WC()->cart->add_to_cart( $product_id, $quantity, $variation_id, $variation );
			// if(!$added) {
			// 	echo html_entity_decode( json_encode(WC()->session->get( 'wc_notices'))) ;
			// 	die();
			// }
		}
		

        do_action( 'woocommerce_ajax_added_to_cart', $product_id );

        if($this->get_option('redirectShowMessage')) {
            wc_add_to_cart_message( $product_id );
        }

        // Return fragments
        WC_AJAX::get_refreshed_fragments();

	    die();
	}

	public function add_all_to_cart()
	{
		ob_start();

		$products = isset( $_POST['products'] ) ? array_filter( $_POST['products'] ) : '';
		if(!empty($products) && is_array($products)) {
			foreach ($products as $product) {

			    if(isset($product['quantity']) && !empty($product['quantity'])) {
			    	$quantity = absint($product['quantity']);
			    } else {
			    	$quantity = 1;
			    }

			    if($quantity == 0) {
			    	$quantity = 1;
			    }

			    $product_id        = apply_filters( 'woocommerce_add_to_cart_product_id', absint( $product['id']) );
			    $product = wc_get_product($product_id);

			    // Variable Product
			    if($product->is_type('variation')) {

			    	$variation_id = $product_id;
			    	$product_id = $product->get_parent_id();

					$passed_validation = apply_filters( 'woocommerce_add_to_cart_validation', true, $product_id, $quantity, $variation_id);
					if ( $passed_validation) {
						WC()->cart->add_to_cart( $product_id, $quantity, $variation_id );
					}
				// Simple Product
			    } else {
				    $passed_validation = apply_filters( 'woocommerce_add_to_cart_validation', true, $product_id, $quantity );

					if ( $passed_validation) {
						WC()->cart->add_to_cart( $product_id, $quantity);
					}
			    }
			}
		} else {
			return false;
		}

        do_action( 'woocommerce_ajax_added_to_cart', $product_id );

        if($this->get_option('redirectShowMessage')) {
            wc_add_to_cart_message( $product_id );
        }

        // Return fragments
        WC_AJAX::get_refreshed_fragments();

	    die();
	}

	/**
	 * Get Variation Add to Cart
	 * @author Daniel Barenkamp
	 * @version 1.0.0
	 * @since   1.0.0
	 * @link    https://www.welaunch.io
	 * @param   [type]                       $variation [description]
	 * @return  [type]                                  [description]
	 */
	public function get_quantity($product)
	{
		ob_start();
	
		do_action( 'woocommerce_before_add_to_cart_quantity' );

		woocommerce_quantity_input( array(
			'min_value'   => apply_filters( 'woocommerce_quantity_input_min', $product->get_min_purchase_quantity(), $product ),
			'max_value'   => apply_filters( 'woocommerce_quantity_input_max', $product->get_max_purchase_quantity(), $product ),
			'input_value' => isset( $_POST['quantity'] ) ? wc_stock_amount( $_POST['quantity'] ) : $product->get_min_purchase_quantity(),
		));
		/**
		 * @since 3.0.0.
		 */
		do_action( 'woocommerce_after_add_to_cart_quantity' );

		$html = ob_get_contents();
		ob_end_clean();
		return $html;
	}

	public function get_multiple_quantity($product)
	{
		global $product;

		ob_start();
	
		do_action( 'woocommerce_before_add_to_cart_quantity' );
		
		woocommerce_quantity_input( array(
			'min_value'   => 0,
			'max_value'   => apply_filters( 'woocommerce_quantity_input_max', $product->get_max_purchase_quantity(), $product ),
			'input_value' => 0,
		) );

		/**
		 * @since 3.0.0.
		 */
		do_action( 'woocommerce_after_add_to_cart_quantity' );

		$html = ob_get_contents();
		ob_end_clean();
		return $html;
	}

	/**
	 * Return the current user role
	 * @author Daniel Barenkamp
	 * @version 1.0.0
	 * @since   1.0.0
	 * @link    https://www.welaunch.io
	 * @return  [type]                       [description]
	 */
	private function get_user_role()
	{
		global $current_user;

		$user_roles = $current_user->roles;
		$user_role = array_shift($user_roles);

		return $user_role;
	}
}